Type: Package
Package: modelbased
Title: Estimation of Model-Based Predictions, Contrasts and Means
Version: 0.16.0
Authors@R: 
    c(person(given = "Dominique",
             family = "Makowski",
             role = c("aut", "cre"),
             email = "officialeasystats@gmail.com",
             comment = c(ORCID = "0000-0001-5375-9967")),
      person(given = "Daniel",
             family = "Lüdecke",
             role = "aut",
             email = "d.luedecke@uke.de",
             comment = c(ORCID = "0000-0002-8895-3206")),
      person(given = "Mattan S.",
             family = "Ben-Shachar",
             role = "aut",
             email = "matanshm@post.bgu.ac.il",
             comment = c(ORCID = "0000-0002-4287-4801")),
      person(given = "Indrajeet",
             family = "Patil",
             role = "aut",
             email = "patilindrajeet.science@gmail.com",
             comment = c(ORCID = "0000-0003-1995-6531")),
      person(given = "Rémi",
             family = "Thériault",
             role = "aut",
             email = "remi.theriault@mail.mcgill.ca",
             comment = c(ORCID = "0000-0003-4315-6788")))
Maintainer: Dominique Makowski <officialeasystats@gmail.com>
Description: Implements a general interface for model-based estimations
    for a wide variety of models, used in the computation of
    marginal means, contrast analysis and predictions. For a list of supported models,
    see 'insight::supported_models()'.
License: GPL-3
URL: https://easystats.github.io/modelbased/
BugReports: https://github.com/easystats/modelbased/issues
Depends: R (>= 4.0)
Imports: bayestestR (>= 0.18.1), datawizard (>= 1.3.1), insight (>=
        1.5.1), parameters (>= 0.29.1), graphics, stats, tools, utils
Suggests: afex, BH, betareg, boot, bootES, brglm2, brms, broom, car,
        carData, coda, collapse, correlation, coxme, curl, discovr,
        easystats, effectsize (>= 1.0.0), emmeans (>= 1.10.2), Formula,
        gamm4, gganimate, ggplot2, glmmTMB (>= 1.1.12), httr2, knitr,
        lme4, lmerTest, logspline, MASS, MatchIt, Matrix,
        marginaleffects (>= 0.29.0), mice, mgcv, mvtnorm, nanoparquet,
        nestedLogit, nnet, ordinal, palmerpenguins, performance (>=
        0.14.0), patchwork, pbkrtest, poorman, pscl, RcppEigen,
        Rdatasets, report, rmarkdown, rstanarm, rtdists, RWiener,
        sandwich, scales, see (>= 0.11.0), survival, testthat (>=
        3.2.1), tinyplot (>= 0.6.0), tinytable, vdiffr, withr
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/Needs/check: stan-dev/cmdstanr
Config/Needs/website: easystats/easystatstemplate
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-30 12:11:56 UTC; mail
Author: Dominique Makowski [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5375-9967>),
  Daniel Lüdecke [aut] (ORCID: <https://orcid.org/0000-0002-8895-3206>),
  Mattan S. Ben-Shachar [aut] (ORCID:
    <https://orcid.org/0000-0002-4287-4801>),
  Indrajeet Patil [aut] (ORCID: <https://orcid.org/0000-0003-1995-6531>),
  Rémi Thériault [aut] (ORCID: <https://orcid.org/0000-0003-4315-6788>)
Repository: CRAN
Date/Publication: 2026-06-30 13:20:20 UTC
