# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,estimate_contrasts)
S3method(as.data.frame,estimate_means)
S3method(as.data.frame,estimate_predicted)
S3method(as.data.frame,estimate_slopes)
S3method(describe_nonlinear,data.frame)
S3method(describe_nonlinear,estimate_predicted)
S3method(describe_nonlinear,numeric)
S3method(display,estimate_contrasts)
S3method(display,estimate_grouplevel)
S3method(display,estimate_means)
S3method(display,estimate_predicted)
S3method(display,estimate_slopes)
S3method(display,estimate_smooth)
S3method(display,visualisation_matrix)
S3method(estimate_contrasts,default)
S3method(estimate_contrasts,estimate_predicted)
S3method(estimate_grouplevel,brmsfit)
S3method(estimate_grouplevel,default)
S3method(estimate_grouplevel,stanreg)
S3method(format,estimate_contrasts)
S3method(format,estimate_grouplevel)
S3method(format,estimate_means)
S3method(format,estimate_predicted)
S3method(format,estimate_slopes)
S3method(format,estimate_smooth)
S3method(format,marginaleffects_contrasts)
S3method(format,marginaleffects_means)
S3method(format,marginaleffects_slopes)
S3method(format,summary_estimate_slopes)
S3method(format,visualisation_matrix)
S3method(plot,estimate_contrasts)
S3method(plot,estimate_grouplevel)
S3method(plot,estimate_means)
S3method(plot,estimate_predicted)
S3method(plot,estimate_slopes)
S3method(print,estimate_contrasts)
S3method(print,estimate_grouplevel)
S3method(print,estimate_means)
S3method(print,estimate_predicted)
S3method(print,estimate_slopes)
S3method(print,estimate_smooth)
S3method(print,summary_estimate_slopes)
S3method(print,visualisation_matrix)
S3method(print_html,estimate_contrasts)
S3method(print_html,estimate_grouplevel)
S3method(print_html,estimate_means)
S3method(print_html,estimate_predicted)
S3method(print_html,estimate_slopes)
S3method(print_html,estimate_smooth)
S3method(print_html,visualisation_matrix)
S3method(print_md,estimate_contrasts)
S3method(print_md,estimate_grouplevel)
S3method(print_md,estimate_means)
S3method(print_md,estimate_predicted)
S3method(print_md,estimate_slopes)
S3method(print_md,estimate_smooth)
S3method(print_md,visualisation_matrix)
S3method(reshape_grouplevel,data.frame)
S3method(reshape_grouplevel,default)
S3method(reshape_grouplevel,estimate_grouplevel)
S3method(residualize_over_grid,data.frame)
S3method(residualize_over_grid,default)
S3method(residualize_over_grid,estimate_means)
S3method(residualize_over_grid,estimate_predicted)
S3method(smoothing,data.frame)
S3method(smoothing,numeric)
S3method(standardize,estimate_contrasts)
S3method(standardize,estimate_means)
S3method(standardize,estimate_predicted)
S3method(standardize,estimate_slopes)
S3method(summary,estimate_slopes)
S3method(summary,reshape_grouplevel)
S3method(tinyplot::tinyplot,estimate_grouplevel)
S3method(tinyplot::tinyplot,estimate_means)
S3method(tinyplot::tinyplot,estimate_predicted)
S3method(tinyplot::tinyplot,estimate_slopes)
S3method(unstandardize,estimate_contrasts)
S3method(unstandardize,estimate_means)
S3method(unstandardize,estimate_predicted)
S3method(visualisation_recipe,estimate_grouplevel)
S3method(visualisation_recipe,estimate_means)
S3method(visualisation_recipe,estimate_predicted)
S3method(visualisation_recipe,estimate_slopes)
export(collapse_by_group)
export(describe_nonlinear)
export(display)
export(estimate_contrasts)
export(estimate_expectation)
export(estimate_grouplevel)
export(estimate_link)
export(estimate_means)
export(estimate_prediction)
export(estimate_relation)
export(estimate_slopes)
export(estimate_smooth)
export(find_inversions)
export(get_emcontrasts)
export(get_emmeans)
export(get_emtrends)
export(get_marginalcontrasts)
export(get_marginalmeans)
export(get_marginaltrends)
export(pool_contrasts)
export(pool_predictions)
export(pool_slopes)
export(print_html)
export(print_md)
export(reshape_grouplevel)
export(residualize_over_grid)
export(smoothing)
export(standardize)
export(unstandardize)
export(visualisation_recipe)
export(zero_crossings)
importFrom(datawizard,standardize)
importFrom(datawizard,unstandardize)
importFrom(datawizard,visualisation_recipe)
importFrom(insight,display)
importFrom(insight,print_html)
importFrom(insight,print_md)
