#' \code{modelbased}
#'
#' @title modelbased: Estimation of Model-Based Predictions, Contrasts and Means
#'
#' @description
#'
#' `modelbased` is a package helping with model-based estimations, to
#' easily compute of marginal means, contrast analysis and model predictions.
#'
#' @docType package
#' @aliases modelbased modelbased-package
#' @name modelbased-package
#' @keywords internal
"_PACKAGE"
