#' @export
#' @importFrom datawizard visualisation_recipe
datawizard::visualisation_recipe

#' @export
#' @importFrom datawizard standardize
datawizard::standardize

#' @export
#' @importFrom datawizard unstandardize
datawizard::unstandardize

#' @importFrom insight print_md
#' @export
insight::print_md

#' @importFrom insight print_html
#' @export
insight::print_html

#' @importFrom insight display
#' @export
insight::display
