---
title: "Overview of Vignettes"
output:
  rmarkdown::html_vignette:
vignette: >
  %\VignetteIndexEntry{Overview of Vignettes}
  \usepackage[utf8]{inputenc}
  %\VignetteEngine{knitr::rmarkdown}
editor_options:
  chunk_output_type: console
---

```{r message=FALSE, warning=FALSE, include=FALSE}
library(knitr)
knitr::opts_chunk$set(
  echo = TRUE,
  collapse = TRUE,
  warning = FALSE,
  message = FALSE,
  comment = "#>",
  eval = TRUE
)
```

All package vignettes are available at [https://easystats.github.io/modelbased/](https://easystats.github.io/modelbased/).

## Function Overview

* [Function Documentation](https://easystats.github.io/modelbased/reference/index.html)

## Introductions

### Basics

* [Data grids](https://easystats.github.io/modelbased/articles/visualisation_matrix.html)
* [What are, why use and how to get marginal means](https://easystats.github.io/modelbased/articles/estimate_means.html)
* [Contrast analysis](https://easystats.github.io/modelbased/articles/estimate_contrasts.html)
* [Marginal effects and derivatives](https://easystats.github.io/modelbased/articles/estimate_slopes.html)
* [Mixed effects models](https://easystats.github.io/modelbased/articles/mixed_models.html)

### Interpretation

* [Use a model to make predictions](https://easystats.github.io/modelbased/articles/estimate_response.html)
* [Interpret simple and complex models using the power of Effect Derivatives](https://easystats.github.io/modelbased/articles/derivatives.html)
* [How to use Mixed models to Estimate Individuals' Scores](https://easystats.github.io/modelbased/articles/estimate_grouplevel.html)

### Visualization

* [Plotting estimated marginal means](https://easystats.github.io/modelbased/articles/plotting.html)
* [Plotting estimated marginal means with tinyplot](https://easystats.github.io/modelbased/articles/plotting_tinyplot.html)
* [Visualize effects and interactions](https://easystats.github.io/modelbased/articles/estimate_relation.html)
* [The Modelisation Approach to Statistics](https://easystats.github.io/modelbased/articles/modelisation_approach.html)


## Case Studies

### Workflows

* [Understanding your models](https://easystats.github.io/modelbased/articles/workflow_modelbased.html)
* [Causal inference for observational data](https://easystats.github.io/modelbased/articles/practical_causality.html)
* [Intersectionality analysis using the MAIHDA framework](https://easystats.github.io/modelbased/articles/practical_intersectionality.html)
* [Measuring and comparing absolute and relative inequalities](https://easystats.github.io/modelbased/articles/practical_inequalities.html)
* [Context Effects: How the Environment Shapes the Individual](https://easystats.github.io/modelbased/articles/practical_context_effect.html)
* [Interrupted Time Series Analysis with `modelbased`](https://easystats.github.io/modelbased/articles/practical_its.html)
* [An Introduction to Growth Mixture Models](https://easystats.github.io/modelbased/articles/practical_growthmixture.html)


### Contrasts

* [Contrasts and pairwise comparisons](https://easystats.github.io/modelbased/articles/introduction_comparisons_1.html)
* [User Defined Contrasts and Joint Tests](https://easystats.github.io/modelbased/articles/introduction_comparisons_2.html)
* [Slopes, floodlight and spotlight analysis (Johnson-Neyman intervals)](https://easystats.github.io/modelbased/articles/introduction_comparisons_3.html)
* [Contrasts and comparisons for generalized linear models](https://easystats.github.io/modelbased/articles/introduction_comparisons_4.html)
* [Contrasts and comparisons for zero-inflation models](https://easystats.github.io/modelbased/articles/introduction_comparisons_5.html)
